#include <time.h>
#include <gl/glut.h>
#include <gl/gl.h>
#include <stdlib.h>
#include <stdio.h>

const int WIDTH = 600;
const int HEIGHT = 600;
const int NUMPOINTS = 200000;

// function prototypes
void drawDot(GLint x, GLint y);
int random(int m);
void display(void);
void init(void);

///////////////////  drawDot  //////////////////
void drawDot(GLint x, GLint y) {
	glBegin(GL_POINTS);
	glVertex2i(x, y);
	glEnd();
}

# define NUM_ITERATIONS 30

void plotPixel(GLint x, GLint y, int i) {
    const static float scale = 0.15;        
	// fill array with color palette
	// RB combinations
	if (i < NUM_ITERATIONS/2) {
	  // R -> G
	  glColor3f(1.0 - i*scale, i*scale, 0.0);
	} else if (i < 30) {
	  i -= 15;
	  // G -> B
	  glColor3f(0.0, 1.0 - i*scale, i*scale);
	} else {
	  // convergence color: blue
	  glColor3f(0.0, 0.0, 1.0);
	}
    drawDot(x,y);
}

static float cx = 0;
static float cy = 0;

void drawJulia(void) {
	 double t, x, y;
  	 double i,j;
  	 int k;

     glClear(GL_COLOR_BUFFER_BIT); // clear the screen

     for (i = -2; i < +2; i+= 0.005) {
        for (j = -2; j < +2; j+= 0.005) {
            x = i;
            y = j;
            
            // test for divergence
  			for (k=0; k < NUM_ITERATIONS; k++) {
			     // apply formula: z^2 +cx +cyi = x^2 - y^2 + 2xyi +cx +cyi
			     //                             = (x^2 - y^2 + cx) + (2xy + cy)i
  			     t = x*x-y*y+cx;
  			     y = 2*x*y+cy;
  			     x = t;
			     // check whether this new point (x,y) is within the bounding circle
  			     if (x*x+y*y >= 4)
  			         break;
  			}
  			
            plotPixel((GLint) ((i * WIDTH/4) + WIDTH/2),
                      (GLint) ((j * HEIGHT/4) + HEIGHT/2), k);
        }
     }
     cx += 0.1;
     cy += 0.1;
	 glutSwapBuffers(); // force drawing to the screen
}

///////////////////  init  //////////////////
void init(void) {
	glClearColor(0.0, 0.0, 0.0, 0.0); // set black background color
	glColor3f(0.2f, 1.0f, 0.2f);      // set the drawing color to green
	glPointSize(1.0);                 // a dot is a 1 x 1 pixel
	glMatrixMode(GL_PROJECTION);      // apply subsequent matrix operations to the projection matrix
	glLoadIdentity();                 // set the projection matrix to be the identity matrix
	gluOrtho2D(0.0, WIDTH, 0.0, HEIGHT); // set up a 2D orthographic viewing region
}

///////////////////  main  //////////////////
void main(int argc, char** argv) {
	glutInit(&argc, argv); // initialize the toolkit
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);  //use a single buffer and RGB color system
	glutInitWindowSize(WIDTH, HEIGHT);  // set window size
	glutInitWindowPosition(0, 0); // set the window position on the screen
	glutCreateWindow("Iterated Function System");  // open the screen window

	init();  // additional initializations are necessary
	srand( (unsigned)time(NULL) );  // seed the random number generator
	//glutDisplayFunc(display);  // register the callback functions
	glutDisplayFunc(drawJulia);
	glutIdleFunc(drawJulia);
	glutMainLoop();  // go into a perpetual loop
}
