/* Author: Brian J. Ross
   October 1999
   
   Adapted by Tom Van Cutsem
   
   Draw 3D Sierpinski Pyramids to level i. 
   The unit pyramid is based upon the local origin sitting in its middle.
   The recursive cases presume the sub-pyramids are scaled 1/2, and then
   translated to new loci around bigger triangle position. */

//#include <GL/gl.h>
#include <stdlib.h>
#include <GL/glut.h>

// position and rotation speed variables.
float X_Rot   = 0.9f;
float Y_Rot   = 0.0f;
float X_Speed = 0.0f;
float Y_Speed = 0.0f;
float Z_Off   =-5.0f;

void recursive_tri(int i) {
	if (i <= 1) {  /* Base case: draw a pyramid */
	   glBegin(GL_TRIANGLES);
	      glColor3f(1.0f, 0.0f, 0.0f);
	      glVertex3f(-80.0, -80.0, 80.0);
	      glColor3f(0.0f, 1.0f, 0.0f);
	      glVertex3f(80.0, -80.0, 80.0);
	      glColor3f(0.0f, 0.0f, 1.0f);
	      glVertex3f(0.0, 80.0, 0.0);

		  glColor3f(1.0f, 0.0f, 0.0f);
	      glVertex3f(80.0, -80.0, 80.0);
	      glColor3f(0.0f, 1.0f, 0.0f);
	      glVertex3f(0.0, -80.0, -80.0);
	      glColor3f(0.0f, 0.0f, 1.0f);
	      glVertex3f(0.0, 80.0, 0.0);

	      glColor3f(1.0f, 0.0f, 0.0f);
	      glVertex3f(0.0, -80.0, -80.0);
	      glColor3f(0.0f, 1.0f, 0.0f);
	      glVertex3f(-80.0, -80.0, 80.0);
	      glColor3f(0.0f, 0.0f, 1.0f);
	      glVertex3f(0.0, 80.0, 0.0);

		  glColor3f(1.0f, 0.0f, 0.0f);
	      glVertex3f(-80.0, -80.0, 80.0);
	      glColor3f(0.0f, 1.0f, 0.0f);
	      glVertex3f(80.0, -80.0, 80.0);
	      glColor3f(0.0f, 0.0f, 1.0f);
	      glVertex3f(0.0, -80.0, -80.0);
	   glEnd();
	} else {
	   --i;

	   glPushMatrix();	 /* draw lower front left */
	   glTranslatef(-40.0, -40.0, 40.0);
	   glScalef(0.5, 0.5, 0.5);
	   recursive_tri(i);
	   glPopMatrix();

	   glPushMatrix();	 /* draw lower front right */
	   glTranslatef(40.0, -40.0, 40.0);
	   glScalef(0.5, 0.5, 0.5);
	   recursive_tri(i);
	   glPopMatrix();

	   glPushMatrix();	 /* draw lower back middle */
	   glTranslatef(0.0, -40.0, -40.0);
	   glScalef(0.5, 0.5, 0.5);
	   recursive_tri(i);
	   glPopMatrix();

	   glPushMatrix();	 /* draw top-middle */
	   glTranslatef(0.0, 40.0, 0.0);
	   glScalef(0.5, 0.5, 0.5);
	   recursive_tri(i);
	   glPopMatrix();
	}
}

// depth variable

int recDepth = 6;

void drawtri() {
	 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	 
	 // Need to manipulate the ModelView matrix to move our model around.
     glMatrixMode(GL_MODELVIEW);

     // Reset to 0,0,0; no rotation, no scaling.
     glLoadIdentity();

     // Move the object back from the screen.
     glTranslatef(0.0f,0.0f,Z_Off);

     // Rotate the calculated amount.
     glRotatef(X_Rot,1.0f,0.0f,0.0f);
     glRotatef(Y_Rot,0.0f,1.0f,0.0f);
	 
	 // draw the sierpinski triangles
	 recursive_tri(recDepth);
	 
	 // done drawing, show it
	 glutSwapBuffers();
	 
	 // Now let's do the motion calculations.
     X_Rot+=X_Speed;
     Y_Rot+=Y_Speed;
}

void keypressed(unsigned char key, int x, int y) {
    switch (key) {
       case 27: //escape
         exit(0);
       case '+':
         recDepth++;
         break;
       case '-': {
         recDepth--;
         break;
       }
    }
}

void specialKeyPressed(int key, int x, int y) {
   switch (key) {
   case GLUT_KEY_PAGE_UP: // move into the distance.
      Z_Off -= 0.05f;
      break;

   case GLUT_KEY_PAGE_DOWN: // move closer.
      Z_Off += 0.05f;
      break;

   case GLUT_KEY_UP: // decrease x rotation speed;
      X_Speed -= 0.01f;
      break;

   case GLUT_KEY_DOWN: // increase x rotation speed;
      X_Speed += 0.01f;
      break;

   case GLUT_KEY_LEFT: // decrease y rotation speed;
      Y_Speed -= 0.01f;
      break;

   case GLUT_KEY_RIGHT: // increase y rotation speed;
      Y_Speed += 0.01f;
      break;
    }
}

int main(int argc, char **argv) {
	glutInit(&argc, argv);
	glutInitWindowSize(600, 600);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutCreateWindow("3D Sierpinski Pyramids");

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-125.0, 125.0, -125.0, 125.0, -125.0, 125.0);
 	glRotatef(15.0, 1.0, 0.0, 0.0);
	glRotatef(45.0, 0.0, 1.0, 0.0);
       
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glPushMatrix();
	glEnable(GL_DEPTH_TEST);

	glClearColor(0.0, 0.0, 0.0, 1.0);  /* black */

    glutKeyboardFunc(keypressed);
    glutSpecialFunc(specialKeyPressed);
	glutDisplayFunc(drawtri);
	glutIdleFunc(drawtri);
	
	glutMainLoop();
	return 0;
}